@@ -6,7 +6,7 @@ Core implementation of the module
 
 import math
 import numpy as np
-import matplotlib
+import matplotlib as mpl
 import ROOT
 import re
 import copy
@@ -42,8 +42,8 @@ class Hist2D(object):
             self.nbinsy = ny = self.binlabelsy.index('')
             self.binlabelsy = self.binlabelsy[:ny]
         self.content = array([array([hist.GetBinContent(i, j)
-                                     for j in range(1, ny + 1)])
-                              for i in range(1, nx + 1)])
+                                     for i in range(1, nx + 1)])
+                              for j in range(1, ny + 1)])
         self.xedges = array([hist.GetXaxis().GetBinLowEdge(i)
                              for i in range(1, nx + 2)])
         self.yedges = array([hist.GetYaxis().GetBinLowEdge(i)
@@ -68,7 +68,7 @@ class Hist2D(object):
         plot = plt.imshow(self.content, interpolation='nearest',
                           extent=[self.xedges[0], self.xedges[-1],
                                   self.yedges[0], self.yedges[-1]],
-                          aspect='auto', **kwargs)
+                          aspect='auto', origin='lower', **kwargs)
         return plot
     def colz(self, **kwargs):
         """Draw a colored box plot with a colorbar using pyplot.imshow."""
@@ -76,7 +76,8 @@ class Hist2D(object):
         plt.colorbar(plot)
         return plot
     def box(self, maxsize=40, **kwargs):
-        """Draw a box plot with size indicating content using pyplot.scatter
+        """
+        Draw a box plot with size indicating content using pyplot.scatter.
 
         The data will be normalized, with the largest box using a marker of
         size maxsize (in points).
@@ -87,6 +88,7 @@ class Hist2D(object):
         maxvalue = np.max(self.content)
         sizes = self.content.flatten() / maxvalue * maxsize
         plot = plt.scatter(x, y, sizes, marker='s', **kwargs)
+        return plot
         
 class Hist(object):
     """A container to hold the parameters from a ROOT histogram."""
@@ -123,11 +125,37 @@ class Hist(object):
         if xlabel: self.xlabel = _parse_latex(xlabel, replace)
         if ylabel: self.ylabel = _parse_latex(ylabel, replace)
         self.label = _parse_latex(label, replace)
+    def __getitem__(self, index):
+        """Return the contents of the indexth bin."""
+        return self.y[index]
+    def __setitem__(self, index, value):
+        """Set the contents of the indexth bin."""
+        self.y[index] = value
+    def __len__(self):
+        """Return the number of bins."""
+        return self.nbins
+    def __iter__(self):
+        """Return the contents of the bins."""
+        return iter(self.y)
     def show_titles(self):
         """Add the titles defined in the ROOT histogram to the figure."""
         plt.title(self.title)
         plt.xlabel(self.xlabel)
         plt.ylabel(self.ylabel)
+    def delete_bin(self, index):
+        """
+        Delete a the contents of a bin, sliding all the other data one bin to
+        the left.  This can be useful for histograms with labeled bins.
+        """
+        self.nbins -= 1
+        self.xedges = self.xedges[:-1]
+        self.x = self.x[:-1]
+        self.width = self.width[:-1]
+        self.y = np.delete(self.y, index)
+        self.xerr = np.delete(self.xerr, index)
+        self.yerr = np.delete(self.yerr, index)
+        if self.binlabels:
+            self.binlabels.pop(index)
     def hist(self, **kwargs):
         """
         Generate a matplotlib hist figure.
@@ -157,17 +185,49 @@ class Hist(object):
             plt.xticks(np.arange(self.nbins), self.binlabels)
         plt.xlim(self.xedges[0], self.xedges[-1])
         return errorbar
-    def bar(self, yerr=False, **kwargs):
+    def errorbarh(self, xerr=False, yerr=False, **kwargs):
+        """
+        Generate a horizontal matplotlib errorbar figure.
+
+        All additional keyword arguments will be passed to pyplot.errorbar.
+        """
+        if xerr: kwargs['xerr'] = self.yerr
+        if yerr: kwargs['yerr'] = self.xerr
+        _load_pyplot()
+        errorbar = plt.errorbar(self.y, self.x, label=self.label, **kwargs)
+        if self.binlabels is not None:
+            plt.yticks(np.arange(self.nbins), self.binlabels)
+        plt.ylim(self.xedges[0], self.xedges[-1])
+        return errorbar
+    def bar(self, xerr=False, yerr=False, **kwargs):
         """
         Generate a matplotlib bar figure.
 
         All additional keyword arguments will be passed to pyplot.bar.
         """
+        if xerr: kwargs['xerr'] = self.xerr
         if yerr: kwargs['yerr'] = self.yerr
         _load_pyplot()
         bar = plt.bar(self.xedges[0:-1], self.y, self.width, label=self.label,
                       **kwargs)
         return bar
+    def barh(self, xerr=False, yerr=False, yoffset=0., width=0.8, **kwargs):
+        """
+        Generate a horizontal matplotlib bar figure.
+
+        All additional keyword arguments will be passed to pyplot.barh.
+        """
+        if xerr: kwargs['xerr'] = self.yerr
+        if yerr: kwargs['yerr'] = self.xerr
+        _load_pyplot()
+        xcontent = [self.xedges[i] + self.width[i] * yoffset
+                    for i in range(len(self.xedges) - 1)]
+        width = self.width * width
+        barh = plt.barh(xcontent, self.y, width, label=self.label,
+                       **kwargs)
+        if self.binlabels is not None:
+            plt.yticks(np.arange(self.nbins), self.binlabels)
+        return barh
     def TH1F(self, name=""):
         """Return a ROOT.TH1F object with the contents of this Hist"""
         th1f = ROOT.TH1F(name, "", self.nbins, self.xedges)
@@ -208,7 +268,24 @@ class HistStack(object):
             for hist in hists:
                 self.add(hist, replace)
     def __getitem__(self, index):
+        """Return the indexth hist in the stack."""
         return self.hists[index]
+    def __setitem__(self, index, value):
+        """Replace the indexth hist in the stack with value."""
+        self.hists[index] = value
+    def __len__(self):
+        """Return the number of hists in the stack."""
+        return len(self.hists)
+    def __iter__(self):
+        return iter(self.hists)
+    def max(self):
+        """Return the value of the highest bin of all the hists in the stack."""
+        maxes = [max(x) for x in self.hists]
+        return max(maxes)
+    def min(self):
+        """Return the value of the lowest bin of all the hists in the stack."""
+        mins = [min(x) for x in self.hists]
+        return min(mins)
     def show_titles(self):
         self.hists[-1].show_titles()
     def add(self, hist, replace=None, **kwargs):
@@ -290,20 +367,67 @@ class HistStack(object):
             if not bottom: bottom = [0. for i in range(self.hists[0].nbins)]
             bottom = [sum(pair) for pair in zip(bottom, hist.y)]
         return plots
-    def errorbar(self, **kwargs):
+    def barh(self, **kwargs):
+        """
+        Make a horizontal clustered matplotlib bar plot.
+
+        Any additional keyword arguments will be passed to pyplot.barh.
+        """
+        plots = []
+        fullwidth = 0.8
+        spacer = (1. - fullwidth) / 2
+        width = 0.8 / len(self.hists)
+        for i, hist in enumerate(self.hists):
+            if self.title  is not None: hist.title  = self.title
+            if self.xlabel is not None: hist.ylabel = self.xlabel
+            if self.ylabel is not None: hist.xlabel = self.ylabel
+            all_kwargs = copy.copy(kwargs)
+            all_kwargs.update(self.kwargs[i])
+            all_kwargs.pop('fmt', None)
+            bar = hist.barh(yoffset=width*i + spacer, width=width, **all_kwargs)
+            plots.append(bar)
+        return plots
+    def errorbar(self, offset=False, **kwargs):
         """
         Make a matplotlib errorbar plot, with all Hists in the stack overlaid.
 
-        Any additional keyword arguments will be passed to pyplot.errorbar.
+        Passing 'offset=True' will slightly offset each dataset so overlapping
+        errorbars are still visible.  Any additional keyword arguments will
+        be passed to pyplot.errorbar.
         """
         plots = []
+        _load_pyplot()
         for i, hist in enumerate(self.hists):
             if self.title  is not None: hist.title  = self.title
             if self.xlabel is not None: hist.xlabel = self.xlabel
             if self.ylabel is not None: hist.ylabel = self.ylabel
             all_kwargs = copy.copy(kwargs)
             all_kwargs.update(self.kwargs[i])
-            errorbar = hist.errorbar(**all_kwargs)
+            transform = plt.gca().transData
+            if offset:
+                index_offset = (len(self.hists) - 1)/2.
+                pixel_offset = 1./72 * (i - index_offset)
+                transform = transforms.ScaledTranslation(
+                    pixel_offset, 0, plt.gcf().dpi_scale_trans)
+                transform = plt.gca().transData + transform
+            errorbar = hist.errorbar(transform=transform, **all_kwargs)
+            plots.append(errorbar)
+        return plots
+    def errorbarh(self, **kwargs):
+        """
+        Make a horizontal matplotlib errorbar plot, with all Hists in the
+        stack overlaid.
+
+        Any additional keyword arguments will be passed to pyplot.errorbar.
+        """
+        plots = []
+        for i, hist in enumerate(self.hists):
+            if self.title  is not None: hist.title  = self.title
+            if self.xlabel is not None: hist.ylabel = self.xlabel
+            if self.ylabel is not None: hist.xlabel = self.ylabel
+            all_kwargs = copy.copy(kwargs)
+            all_kwargs.update(self.kwargs[i])
+            errorbar = hist.errorbarh(**all_kwargs)
             plots.append(errorbar)
         return plots
     def bar(self, **kwargs):
@@ -344,9 +468,12 @@ class RootFile:
         """Return a Hist object from the given path within this file."""
         return Hist(self.file.Get(object_name))
 
-def ls(directory=""):
+def ls(directory=None):
     """Return a python list of ROOT object names from the given directory."""
-    keys = ROOT.gDirectory.GetDirectory(directory).GetListOfKeys()
+    if directory == None:
+        keys = ROOT.gDirectory.GetListOfKeys()
+    else:
+        keys = ROOT.gDirectory.GetDirectory(directory).GetListOfKeys()
     key = keys[0]
     names = []
     while key:
@@ -367,8 +494,9 @@ def get(object_name):
 ################ Define additional helping functions
 
 def _load_pyplot():
-    global plt
-    from matplotlib import pyplot as plt
+    global plt, transforms
+    import matplotlib.pyplot as plt
+    import matplotlib.transforms as transforms
 
 def _parse_latex(string, replacements):
     """Modify a string based on a dictionary of replacements."""
@@ -423,7 +551,7 @@ def axes_broken_y(axes, upper_frac=0.5, break_frac=0.02, ybounds=None,
             if loc  == 'left':
                 color = spine.get_edgecolor()
         for x_position in [x1, x2]:
-            line = matplotlib.lines.Line2D(
+            line = mpl.lines.Line2D(
                 [x_position + offset for offset in xoffsets], yvalues,
                 transform=plt.gcf().transFigure, clip_on=False,
                 color=color)
@@ -490,7 +618,7 @@ def prepare_efficiency(axes, lower_bound=0.69):
 ## ##         if position == 'bottom': j = 0
 ## ##         if position == 'top': j = 1
 ## ##         for i in range(2):
-## ##             line = matplotlib.lines.Line2D(
+## ##             line = mpl.lines.Line2D(
 ## ##                 [plt.xlim()[i] - xwidth, plt.xlim()[i] + xwidth],
 ## ##                 [plt.ylim()[j] - ywidth, plt.ylim()[j] + ywidth],
 ## ##                 linewidth=1, color='black', clip_on=False)
@@ -498,5 +626,6 @@ def prepare_efficiency(axes, lower_bound=0.69):
 
 
 if __name__ == "__main__":
+    # no doctest stuff yet implemented; maybe in future it will be
     import doctest
     doctest.testmod()
