@@ -9,6 +9,7 @@ __version__ = '1.4'
 ## Created by Jeff Klukas (klukas@wisc.edu), November 2009
 ## Updated March 2010
 
+
 ######## Import python libraries #############################################
 
 import sys
@@ -18,6 +19,13 @@ import math
 import os
 import re
 
+try:
+    import multiprocessing
+    from Queue import Empty
+    use_multiprocessing = True
+except ImportError:
+    use_multiprocessing = False
+
 ## If we actually plan to do something other than show the help menu, 
 ## import the PyROOT package
 if '-h' not in sys.argv and len(sys.argv) > 1:
@@ -32,6 +40,75 @@ if '-h' not in sys.argv and len(sys.argv) > 1:
         ROOT.gROOT.Macro('rootlogon.C')
 
 
+######## Define styles #######################################################
+
+default_styles=r"""
+## When using matplotlib, all text from the first column will be replaced
+## with the corresponding text in the second column for titles and labels;
+## this is necessary to correctly transform ROOT's special LaTeX sytax into
+## proper LaTeX sytax for use by matplotlib.
+
+## By default, matplotlib will render only symbols between $'s as TeX, but if
+## you enable the 'text.usetex' matplotlibrc setting, then everything is handled
+## by the LaTeX engine on your system, in which case you can go wild with TeX.
+
+replace = [
+    # some defaults that should work for most cases
+    (' pt '    , r' $p_\mathrm{T}$ '),
+    ('pT '     , r'$p_\mathrm{T}$ '),
+    (' pT'     , r' $p_\mathrm{T}$'),
+    ('p_{T}'   , r'$p_\mathrm{T}$'),
+    ('#eta'    , r'$\eta$'),
+    ('#phi'    , r'$\phi$'),
+    ('fb^{-1}' , r'$\mathrm{fb}^{-1}$'),
+    ('pb^{-1}' , r'$\mathrm{pb}^{-1}$'),
+    ('<'       , r'$<$'),
+    ('>'       , r'$>$'),
+    ('#'       , r''),
+    ]
+
+colors = [
+    ## a default set of contrasting colors the author happens to like
+    ( 82, 124, 219), # blue
+    (212,  58, 143), # red
+    (231, 139,  77), # orange
+    (145,  83, 207), # purple
+    (114, 173, 117), # green
+    ( 67,  77,  83), # dark grey
+    ]
+
+markers_matplotlib = [
+    'o', 's', '^', 'x', '*', 'D', 'h', '1'
+    ]
+
+markers_root = [
+    ## This default set mirrors the matplotlib markers above
+     4, # circle 
+    25, # square
+    26, # triangle
+     5, # x
+    30, # five-pointed star
+    27, # diamond
+    28, # cross
+     3, # asterisk
+    ]
+
+## Define the size of the legend in ROOT
+legend_width = 0.22        # Fraction of canvas width
+legend_entry_height = 0.04 # Fraction of canvas height
+max_legend_height = 0.3    # Fraction of canvas height
+
+## For more complex needs, you may want to specify a list of files along with
+## more descriptive names for the legend using the filenames option.  When
+## enabled, any filenames on the command line will be ignored.
+
+## filenames = [
+##     ('histTTbar.root', r'$\bar{t}t$'),
+##     ('histZmumu.root', r'$Z\rightarrow\mu\mu$'),
+##     ]
+"""
+
+
 ######## Define classes and generators #######################################
 
 class RootFile:
@@ -65,12 +142,14 @@ def process_directory(path, files):
     dir_to_make = '%s/%s' % (options.plot_dir, path)
     keys = files[0].file.GetDirectory(path).GetListOfKeys()
     key = keys[0]
+    plots_to_make = []
     while key:
         obj = key.ReadObj()
         key = keys.After(key)
-        new_path = '%s/%s' % (path, obj.GetName())
+        name = obj.GetName()
+        new_path = '%s/%s' % (path, name)
         if obj.IsA().InheritsFrom('TDirectory'):
-            process_directory(new_path, files)
+            plots_to_make += process_directory(new_path, files)
         #### If obj is a desired histogram, process it
         matches_path = options.regex.search(new_path)
         is_1D_histogram = (obj.IsA().InheritsFrom('TH1') and
@@ -79,17 +158,37 @@ def process_directory(path, files):
         if (matches_path and is_1D_histogram):
             if not os.path.exists(dir_to_make):
                 os.makedirs(dir_to_make)
-            if options.use_matplotlib:
-                process_hist_matplotlib(path, new_path, files, obj)
-            else:
-                process_hist(path, new_path, files, obj)
+            counter = next_counter() # used for page numbers
+            plots_to_make += [(path, name, counter)]
+    return plots_to_make
 
+#### Handle multiprocessing
+def plotting_worker(queue):
+    """Handle a single plot, communicating with the queue."""
+    while True:
+        try:
+            args = queue.get(timeout=5)
+        except Empty:
+            break
+        process_hist(*args)
+        queue.task_done()
 
 #### This is where all the plotting actually happens
-def process_hist(path, new_path, files, obj):
-    """Overlay all the instances of this plot and apply the options."""
-    counter = next_counter() # used for page numbers
-    name = obj.GetName()
+def process_hist(path, name, counter):
+    if options.mpl:
+        process_hist_matplotlib(path, name, counter)
+    else:
+        process_hist_root(path, name, counter)
+
+def process_hist_root(path, name, counter):
+    """Overlay all the instances of this plot in ROOT and apply the options."""
+    try:
+        files = [RootFile(filename) for filename in options.filenames]
+    except TypeError:
+        files = [RootFile(filename, displayname) for filename, displayname
+                 in options.filenames]
+    global canvas
+    canvas = ROOT.TCanvas()
     hist = files[0].file.GetDirectory(path).Get(name)
     title = hist.GetTitle()
     x_title = hist.GetXaxis().GetTitle()
@@ -112,8 +211,8 @@ def process_hist(path, new_path, files, obj):
     if options.area_normalize or (options.sticky and 'Norm' in name):
         y_title = "Fraction of Events in Bin"
     if options.normalize:
-        if type(options.normalize) == type(int):
-            file_name = files[options.normalize_to_file - 1].name
+        if type(options.normalize) == type(int()):
+            file_name = files[options.normalize - 1].name
             y_title = "Events Normalized to %s" % file_name
         else:
             y_title = "Events Scaled by %d" % options.normalize
@@ -134,13 +233,11 @@ def process_hist(path, new_path, files, obj):
                 hist.SetBinError(j + 1, max(up[j], down[j]))
         color = ROOT.TColor.GetColor(*options.colors[i][0:3])
         hist.SetLineColor(color)
-#         if options.fill:
-#             r, g, b = plot_colors_rgb[i % len(colors)]
-#             #fill_color = ROOT.TColor.GetColor(r * 1.2, g * 1.2, b * 1.2)
-#             fill_color = color
-#             hist.SetFillColor(fill_color)
-#             hist.SetFillStyle(1001)
-#             print 'Hist ', hist.GetFillColor()
+        if options.fill or (options.data and (i+1) != options.data):
+            #hist.SetLineColor(ROOT.kBlack)
+            fill_color = color
+            hist.SetFillColor(fill_color)
+            hist.SetFillStyle(1001)
         if options.markers:
             hist.SetMarkerColor(color)
             hist.SetMarkerStyle(options.marker_styles[i])
@@ -158,11 +255,11 @@ def process_hist(path, new_path, files, obj):
             if integral: hist.Scale(1. / integral)
         hists.append(hist)
     if options.normalize:
-        if type(options.normalize) == type(int):
-            integral = hists[options.normalize_to_file - 1].Integral()
+        if type(options.normalize) == type(int()):
+            integral = hists[options.normalize - 1].Integral()
             if integral:
                 for hist in hists:
-                    hist.Scale(hist.Integral() / integral)
+                    hist.Scale(integral / hist.Integral())
         else:
             hist.Scale(options.normalize)
     #### Combine hists in a THStack and draw
@@ -181,8 +278,9 @@ def process_hist(path, new_path, files, obj):
         print "Invalid legend parameter for ROOT output!"
         sys.exit(1)
     legend = ROOT.TLegend(*legend_loc)
-    for hist in hists:
-        stack.Add(hist)
+    for i, hist in enumerate(hists):
+        if (i+1) != options.data:
+            stack.Add(hist)
         legend.AddEntry(hist)
     stack.Draw(options.draw)
     stack.GetXaxis().SetTitle(x_title)
@@ -199,6 +297,14 @@ def process_hist(path, new_path, files, obj):
         stack.Draw(options.draw + 'e')
         stack.SetMaximum(1.)
         stack.SetMinimum(0.)
+    if options.data:
+        hists[options.data - 1].SetLineColor(ROOT.kBlack)
+        hists[options.data - 1].SetMarkerColor(ROOT.kBlack)
+        hists[options.data - 1].SetMarkerStyle(4) # circle
+        hists[options.data - 1].SetMarkerSize(ROOT.gStyle.GetMarkerSize())
+        if hists[options.data - 1].GetMaximum() > stack.GetMaximum():
+            stack.SetMaximum(hists[options.data - 1].GetMaximum() * 1.2)
+        hists[options.data - 1].Draw('same e')
     if options.numbering:
         display_page_number(counter)
     if options.overflow or (options.sticky and 'Overflow' in name):
@@ -209,10 +315,13 @@ def process_hist(path, new_path, files, obj):
     save_plot(stack, options.plot_dir, path, name, counter)
 
 #### This is where all the plotting actually happens
-def process_hist_matplotlib(path, new_path, files, obj):
+def process_hist_matplotlib(path, name, counter):
     """Overlay all the instances of this plot and output using matplotlib."""
-    counter = next_counter() # used for page numbers
-    name = obj.GetName()
+    try:
+        files = [RootFile(filename) for filename in options.filenames]
+    except TypeError:
+        files = [RootFile(filename, displayname) for filename, displayname
+                 in options.filenames]
     xpos = options.size.find('x')
     width = float(options.size[:xpos])
     height = float(options.size[xpos+1:])
@@ -253,15 +362,29 @@ def process_hist_matplotlib(path, new_path, files, obj):
             hist.y /= sum(hist.y)
         if options.markers: fmt = options.marker_styles[i]
         else:               fmt = 'o'
-        stack.add(hist, label=file.name, color=options.colors[i], fmt=fmt)
+        if options.data and (i+1) == options.data:
+            datahist = hist
+            datahist.label = file.name
+        else:
+            stack.add(hist, label=file.name, color=options.colors[i], fmt=fmt)
     if (options.errorbar):
-        plots = stack.errorbar(yerr=True)
+        if options.data:
+            try:
+                stack.hist(histtype='stepfilled')
+            except ValueError:
+                pass
+            datahist.errorbar(yerr=True, fmt='o', color='k')
+        else:
+            plots = stack.errorbar(yerr=True)
     if options.stack:
         stack.barstack()
     if options.bar:
         stack.bar(alpha=0.5)
     if options.hist:
-        stack.hist(alpha=0.5, histtype='stepfilled')
+        try:
+            stack.hist(alpha=0.5, histtype='stepfilled')
+        except ValueError:
+            pass
     if options.efficiency:
         plt.ylim(0, 1)
     if options.logx: ax.set_xscale('log')
@@ -273,7 +396,8 @@ def process_hist_matplotlib(path, new_path, files, obj):
     if options.legend != 'None':
         plt.legend(numpoints=1, loc=options.legend)
     output_file_name = '%s/%s/%s' % (options.plot_dir, path, name)
-    plt.savefig(output_file_name, transparent=options.transparent)
+    plt.savefig(output_file_name, transparent=options.transparent,
+                dpi=options.dpi)
     if options.ext == "pdf":
         shutil.copy("%s.pdf" % output_file_name, "%.3i.pdf" % counter)
     report_progress(counter)
@@ -397,11 +521,18 @@ def add_options_from_configuration(options):
                 attr = getattr(config, attribute)
                 setattr(options, attribute, attr)
         return options
-    try:
-        from root2matplot import overlayHists_config as default_config
-        options = append_to_options(default_config, options)
-    except ImportError:
-        pass
+    f = open('default_overlay_config.py', 'w')
+    f.write(default_styles)
+    f.close()
+    if options.config:
+        os.rename('default_overlay_config.py', 'overlayHists_config.py')
+        print "Wrote overlayHists_config.py to the current directory"
+        sys.exit(0)
+    sys.path.insert(0, '')
+    import default_overlay_config
+    options = append_to_options(default_overlay_config, options)
+    os.remove('default_overlay_config.py')
+    os.remove('default_overlay_config.pyc')
     sys.path.insert(0, '')
     for f in options.config_files:
         user_config = __import__(f[:-3])
@@ -410,8 +541,13 @@ def add_options_from_configuration(options):
 
 def process_options(options):
     nfiles = len(options.filenames)
-    options.use_matplotlib = (options.stack or options.errorbar or
-                              options.bar or options.hist)
+    mpl_option_activated = (options.stack or options.errorbar or
+                            options.bar or options.hist)
+    if options.mpl:
+        if not mpl_option_activated:
+            options.errorbar = True
+    else:
+        options.mpl = mpl_option_activated
     try:
         global matplotlib
         import matplotlib
@@ -423,11 +559,12 @@ def process_options(options):
             matplotlib.use('PS')
         imported_matplotlib = True
     except ImportError:
-        print "Unable to load matplotlib"
-        if options.use_matplotlib:
+        if options.mpl:
+            print("Unable to load matplotlib, "
+                  "which is required for these options")
             sys.exit(1)
         imported_matplotlib = False
-    if options.use_matplotlib:
+    if options.mpl:
         try:
             global r2m, np, plt
             import root2matplot as r2m
@@ -435,7 +572,8 @@ def process_options(options):
             from matplotlib import pyplot as plt
             options.marker_styles = options.markers_matplotlib
         except ImportError:
-            print "Unable to load root2matplot"
+            print("Unable to load root2matplot, "
+                  "which is required for operation with matplotlib")
             sys.exit(1)
     else:
         options.marker_styles = options.markers_root
@@ -466,41 +604,41 @@ def process_options(options):
 ######## Define the main program #############################################
 
 def main():
-    usage="""usage: %prog [options] file1.root file2.root file3.root ...
+    usage="""usage: %prog [options] [style_config.py] [file1.root ...]
 
 Documentation: http://packages.python.org/root2matplot/
 
 Function: overlays corresponding histograms from several files, dumping the
   images into a directory structure that mirrors that of the ROOT file and,
-  if output format is pdf, also merging all images into a single file;
-  most style options can be controlled from your rootlogon.C macro; advanced
-  operation using configuration files is described in the full online
-  documentation.
+  if output format is pdf, also merging all images into a single file.
+  Most style options can be controlled from your rootlogon.C macro.
 
-Note: By default, this script will work completely in ROOT.  If you use the
+Power users: advanced operation using configuration files is described in the
+  full online documentation.  This allows you control over colors, styles,
+  names for the legends, and more.
+
+Matplotlib: if you have the matplotlib python plotting library installed on your
+  system, you can produce output in matplotlib.  By activating any of the
   options 'bar', 'errorbar', 'hist', or 'stack', output will be produced in
-  matplotlib.  More complex style attributes like colors and markers used can
-  be configured by including a python configuration file in the command-line
-  arguments."""
+  matplotlib."""
     
     parser = optparse.OptionParser(usage=usage, 
                                    version='%s %s' % ('%prog', __version__))
-    group2 = optparse.OptionGroup(parser,
-                                "options available only with ROOT output")
-    group3 = optparse.OptionGroup(parser,
-                                "options available only with matplotlib output")
+    parser.add_option('--config', action="store_true",
+                      help="Do nothing but write a template configuration file "
+                      "called overlayHists_config.py")
     parser.add_option('-e', '--ext', default='pdf', 
                       help="choose an output extension; default is pdf")
-    group2.add_option('--draw', default='nostack p H',
-                      help="pass DRAW to ROOT's Draw command; default is "
-                      "'nostack p H'; try 'nostack e' for error bars")
     parser.add_option('-m', '--markers', action='store_true', default=False,
                       help="add markers to histograms")
     parser.add_option('-s', '--sticky', action='store_true', default=False,
                       help="enable name-based special plotting options "
                       "(see below)")
-#     parser.add_option('-f', '--fill', action='store_true', default=False,
-#                       help="Fill histograms with a color")
+    parser.add_option('--data', default=0, type='int', metavar='FILENUM',
+                      help="The histogram from file #FILENUM (starting from 1) "
+                      "will be drawn as black datapoints, while all others "
+                      "will be filled, as is the custom for showing data vs. "
+                      "Monte Carlo.")
     parser.add_option('--output', default='overlaidHists', metavar='NAME',
                       help="name of output directory; default is "
                       "'overlaidHists'")
@@ -513,19 +651,8 @@ Note: By default, this script will work completely in ROOT.  If you use the
     parser.add_option('--normalize', metavar='VALUE',
                       help="if integer, normalize to the VALUEth file "
                       "(starting with 1); if float, scale by VALUE")
-    parser.add_option('--stack', action="store_true", default=False,
-                      help="output a matplotlib stacked bar graph")
-    parser.add_option('--errorbar', action="store_true", default=False,
-                      help="output a matplotlib errorbar graph")
-    parser.add_option('--bar', action="store_true", default=False,
-                      help="output a matplotlib bar graph")
-    parser.add_option('--hist', action="store_true", default=False,
-                      help="output a matplotlib hist graph (with solid fill")
-    group3.add_option('--transparent', action="store_true", default=False,
-                      help="use a transparent background for matplotlib "
-                      "output")
     parser.add_option('--colormap', default=None,
-                      help="Select colors from the selected matplotlib "
+                      help="Select colors from the given matplotlib "
                       "colormap rather than the defaults")
     parser.add_option('--ncolors', type=int, default=None,
                       help="The number of colors with which to divide the "
@@ -535,20 +662,20 @@ Note: By default, this script will work completely in ROOT.  If you use the
                       "location codes; for ROOT, you can use 1 (default) for "
                       "upper right or 8 for lower center; "
                       "use 'None' to not show the legend")
-    group3.add_option('--size', default='6x4.5',
-                      help="Define the size of the matplotlib plot as "
-                      "width x height in inches; default is '6x4.5'")
     parser.add_option('--title', default=None,
-                      help="Add TITLE to the title of each plot")
+                      help="Replace the plot titles, or add to them by "
+                      "preceeding with a '+'")
     parser.add_option('--xlabel', default=None,
-                      help="Replace the x-axis label with XLABEL")
+                      help="Replace the x-axis labels, or add to them by "
+                      "preceeding with a '+'")
     parser.add_option('--ylabel', default=None,
-                      help="Replace the y-axis label with YLABEL")
+                      help="Replace the y-axis labels, or add to them by "
+                      "preceeding with a '+'")
     parser.add_option('--efficiency-from', default=None, metavar="DENOM",
                       help="Divide all plots by the histogram in path DENOM")
     group1 = optparse.OptionGroup(
         parser,
-        "special plotting options",
+        "Special plotting options",
         "Use the command line options given below to apply changes to all "
         "plots.  If you only wish to apply an option to a specific plot, "
         "you can use '-s' "
@@ -575,6 +702,34 @@ Note: By default, this script will work completely in ROOT.  If you use the
     group1.add_option('--ratio', action='store_true', default=False,
                       help="'Ratio': display a ratio plot below the normal "
                       "plot")
+    group2 = optparse.OptionGroup(parser,
+                                "Options specific to ROOT (default) output")
+    group2.add_option('--draw', default='nostack p H',
+                      help="pass DRAW to ROOT's Draw command; default is "
+                      "'nostack p H'; try 'nostack e' for error bars")
+    group2.add_option('-f', '--fill', action='store_true', default=False,
+                      help="Histograms will have a color fill")
+    group3 = optparse.OptionGroup(parser,
+                                "Producing output with matplotlib")
+    group3.add_option('--mpl', action="store_true", default=False,
+                      help="produce output in matplotlib; automatically "
+                      "turned on by --stack, --errorbar, --bar, or --hist")
+    group3.add_option('--stack', action="store_true", default=False,
+                      help="output a matplotlib stacked bar graph")
+    group3.add_option('--errorbar', action="store_true", default=False,
+                      help="output a matplotlib errorbar graph")
+    group3.add_option('--bar', action="store_true", default=False,
+                      help="output a matplotlib bar graph")
+    group3.add_option('--hist', action="store_true", default=False,
+                      help="output a matplotlib hist graph (with solid fill)")
+    group3.add_option('--transparent', action="store_true", default=False,
+                      help="use a transparent background")
+    group3.add_option('--size', default='6x4.5',
+                      help="Define the plot size as "
+                      "'width x height' in inches; default is '6x4.5'")
+    group3.add_option('--dpi', default=None,
+                      help="Set the resolution of matplotlib output (default "
+                      "is 100)")
     parser.add_option_group(group2)
     parser.add_option_group(group3)
     parser.add_option_group(group1)
@@ -602,14 +757,25 @@ Note: By default, this script will work completely in ROOT.  If you use the
         files = [RootFile(filename, displayname) for filename, displayname
                  in options.filenames]
     ## if no filenames provided, just display the help message
-    if len(files) == 0:
+    if len(options.filenames) == 0:
         parser.print_help()
         sys.exit(0)
     options = process_options(options)
-    global canvas
-    canvas = ROOT.TCanvas()
     ## here, we decend into the files to start plotting
-    process_directory('', files)
+    plots_to_make = process_directory('', files)
+    if use_multiprocessing:
+        queue = multiprocessing.JoinableQueue()
+        for args in plots_to_make:
+            queue.put(args)
+        for i in range(4):
+            p = multiprocessing.Process(target=plotting_worker,
+                                        name="worker-%i"%i, args=(queue,))
+            p.daemon = True
+            p.start()
+        queue.join()
+    else:
+        for args in plots_to_make:
+            process_hist(*args)
     print ''
     if options.ext == 'pdf':
         merge_pdf()
